@@ -1,4 +1,4 @@
-# pigai, 20-7-7
+# pigai, 20-7-14
 from ipywidgets import widgets,interact, interactive, fixed, interact_manual,Button, Layout,Dropdown,RadioButtons
 from IPython.display import display, clear_output
 from IPython.core.display import HTML,display
@@ -7,15 +7,52 @@ import pandas as pd
 import matplotlib.pyplot as plt
 import seaborn as sns
 import spacy,json,re,os, platform,builtins,requests
-import pygtrie, random, json
+import pygtrie, random, json,redis
 from functools import reduce
 from urllib.parse import quote
+from collections import Counter,defaultdict
 
 nlp		= spacy.load('en_core_web_sm')
 merge_nps	= nlp.create_pipe("merge_noun_chunks")
 postag		= lambda snt: pd.DataFrame([ (t.text, t.tag_) for t in nlp(snt)], columns=['word','pos'])
 tokenize	= lambda snt: " ".join([t.text for t in nlp(snt) if len(t.text.strip())]).strip()
 
+from elasticsearch import Elasticsearch
+es	= Elasticsearch(['es.werror.com']) 
+r	= redis.Redis(host='dev.werror.com', port=8008,  db=0, decode_responses=True) #rkp	= redis.Redis(host='dev.werror.com', port=8008,  db=1, decode_responses=True)
+
+toarr = lambda spair :  [ pair.split(":") for pair in spair.split(",")] #konwledge:30.09,knowlege:14.46,knoeledge:5.67,kownledge:5.43
+tosf  = lambda spair :  [ (w, float(p)) for w,p in [ pair.split(":") for pair in spair.split(",")]]
+tostr = lambda arr : ",".join([ f"{w}:{p}" for w,p in arr])
+
+rdic = redis.Redis(host='dev.werror.com', port=8008, db=3, decode_responses=True)
+def ecdic(pattern='con*ate', wlen=0,limit=10):
+	words = [k for k in rdic.keys(pattern) if wlen <= 0 or len(k) == wlen][0:limit]
+	return [(w,t) for w, t in zip(words, rdic.mget(words))]
+
+def triples(corpus:str='dic', rel:str='dobj', w:str='open', start:int=0, end:int=10): # zrevrange gzjc:dobj_open_~ 0 10
+	cnt = r.get(f"{corpus}:{rel}_{w}_~")
+	return r.zrevrange(f"{corpus}:{rel}_{w}_~", start, end, withscores = True)
+
+def get(corpus:str='gzjc', word:str='book'):
+	return es.get('vocab','gzjc:book:lex')['_source'] # '_source': {'mf': 94, 'arr': 'book:55.32,books:43.62,booked:1.06'}}
+
+def corpus_snt(word, corpus='dic'):
+	d={
+  "query": { 
+    "bool": { 
+      "must": [
+        { "match": { "snt": word }}
+      ],
+      "filter": [ 
+        { "term":  { "src": corpus }}
+      ]
+    }
+  }
+ }
+	return es.search(body=d, index='sentbase')
+#print(corpus_snt("love"))
+
 api_url = "http://rest.wrask.com"	
 rows		= lambda sql, corpus='dic', columns=[]: requests.get(f"{api_url}/kpfts/query/{corpus}", params={'sql':sql}).json() if not columns else pd.DataFrame(rows(sql, corpus, columns=[]), columns=columns)
 mapk		= lambda sql, corpus='dic', columns=[]: {row[0]:row for row in requests.get(f"{api_url}/kpfts/query/{corpus}", params={'sql':sql}).json()}
@@ -49,16 +86,7 @@ def highlight(snt, merge_np= False,  colors={'ROOT':'red', 'VERB':'orange','ADJ'
 	html =  " ".join(arr) 
 	return HTML(html if font_size <=0 else f"<div style='font-size:{font_size}px'>{html}</div>")
 
-wordlist	= lambda: {w for w, in requests.get(f"{api_url}/kpfts/query/wordlist", params={'sql':'select kp from vocab'}).json()}
-wordidf		= lambda: {w:f for w,f in requests.get(f"{api_url}/kpfts/query/wordidf", params={'sql':'select kp,mf from vocab'}).json()}
-spellerr	= lambda w, topk=10: getarr(w, 'spellerr', end=topk).set_axis(['word','prob'], axis='columns', inplace=False) 
-spellerrper	= lambda w: getmf(w, 'spellerr')
-
-ecdic			= lambda pattern='con*ate', wlen=0,limit=10:  pd.DataFrame(rows(
-f"select kp,arr from vocab where kp like '{pattern.replace('*','%')}' and length(kp) = {wlen} limit {limit} " if wlen > 0 else 
-f"select kp,arr from vocab where kp like '{pattern.replace('*','%')}' limit {limit} ",corpus='ecdic'), columns=['word','trans'])
-#print(ecdic(wlen=10))
-
+spellerr	= lambda w, topk=10: tosf(r.hget('spellerr', w))[0:topk]
 parasent	= lambda snt, topk=10,nprobe=10,corpus='dic': pd.DataFrame(requests.get(f'{api_url}/sntvec/search/{corpus}', params={'snt':snt, 'topk':topk,'nprobe':nprobe}).json(), columns=['sid','snt','semdis'])
 cola		= lambda snt: pd.DataFrame(requests.get(f'{api_url}/cola/{snt}').json(), columns=['word','prob']) #http://cluesay.com:7095/cola/I%20love%20you%7CI%20live%20you |[["I love you",0.973],["I live you",0.2679]]
 nextword	= lambda snt, topk=10: requests.get(f'{api_url}/auto/nextword', params={'snt': snt, 'topk':int(topk)}).json()
@@ -88,7 +116,6 @@ def restate(snt='John opened the window.', tense=0, option=[False, False, True,
 random_one	= lambda arr: arr[ int( len(arr) * random.random() )]
 random_word	= lambda c: random_one([w for w, in requests.get(f"{api_url}/kpfts/query/wordlist", params={'sql':f"select kp from vocab where kp like '{c}%' limit 1000"}).json()])
 nextword_if	= lambda snt_prefix, c , topk= 100: [ row for row in nextword(snt_prefix, topk= topk) if c in row[0] and row[0].isalpha() and len(row[0]) > 1]
-#print(nextword_if("we love", 'y'))
 
 def word_to_sent( word, topk = 300  ) :   #sentord, wordence
 	snt = random_word(word[0])
@@ -102,40 +129,29 @@ def word_to_sent( word, topk = 300  ) :   #sentord, wordence
 #print(word_to_sent("family"))
 
 def subwords(w = 'knowledge', minlen=1, editdis=0):
-	if subwords.counter <= 0: 
-		subwords.counter = 1
-		subwords.trie = pygtrie.CharTrie()
-		for w in wordlist(): subwords.trie[w] = len(w)
+	if not hasattr(subwords, 'trie'):
+		setattr(subwords, 'trie', pygtrie.CharTrie()) #subwords.trie = pygtrie.CharTrie()
+		for w in r.hkeys('wordlist'): subwords.trie[w] = len(w)
 	res =[]
 	for i in range(len(w)): res.extend(subwords.trie.prefixes(w[i:]))
 	res.sort(key =lambda ar: ar[1], reverse=True)
 	return list(filter(lambda pair: pair[1] > minlen, res))
-subwords.counter = 0 
 #print(subwords())
 
 def sent_to_word(snt):
-	if sent_to_word.counter <= 0: 
-		sent_to_word.counter = 1
-		sent_to_word.wlist = wordlist()
-	return {w for w in reduce(lambda x,y:[ i+j for i in x for j in y], [ [a for a in word] for word in snt.split(" ")]) if w in sent_to_word.wlist }
-sent_to_word.counter = 0 
+	cands = list({w for w in reduce(lambda x,y:[ i+j for i in x for j in y], [ [a for a in word] for word in snt.lower().split(" ")])})
+	return [cand for cand, hit in zip(cands, r.hmget('wordlist', cands )) if not hit is None]
+#print(sent_to_word("father and mother i love you"))
 
-from spacy.attrs import LOWER
+#from spacy.attrs import LOWER
 def learn(essay = "The quick fox jumped over the lazy dog. I have learned a lot of knowledges."): 
-	if learn.counter <= 0: 
-		learn.counter = 1
-		learn.widf = wordidf()
 	doc = nlp(essay)
-	wc = doc.count_by(LOWER) #nlp.vocab[8566208034543834098].text
-	d = {} 
-	for t in doc: 
-		if not t.text.lower() in df.index and not t.pos_ in ('PUNCT'):
-			d[t.text.lower()] = {'word': t.text, 'lemma':t.lemma_, 'tag': t.tag_, 'pos': t.pos_, 'count': wc.get(nlp.vocab[t.text.lower()].orth,0), 'diff': learn.widf.get(t.text.lower(),0.0), 'isstop':t.is_stop}
-	return d
-learn.counter = 0 
+	wc = Counter([t.text.lower() for t in doc if not t.pos_ in ('PUNCT') and not t.is_stop])  #doc.count_by(LOWER) #nlp.vocab[8566208034543834098].text
+	return [(w, r.zscore('wordidf', w), c) for w, c in wc.items()]
+#print(learn())
 
 if __name__ == '__main__': 
 	#print(trpstar("dobj_open_%").set_axis(['word','prob','sent'], axis='columns', inplace=False) )
 	#print(getarr('book/pos'))
 	#print(word_to_sent('family'))
-	pass
\ No newline at end of file
+	print(nextword_if("we love", 'y'))
